<!DOCTYPE html>
<html>
<head>
    <title>Trace Files</title>
    <style>
        body { font-family: monospace; margin: 40px; }
        h1 { color: #333; }
        ul { list-style-type: none; padding: 0; }
        li { margin: 10px 0; }
        a { color: blue; text-decoration: none; }
        a:hover { text-decoration: underline; }
    </style>
</head>
<body>
    <h1>Trace Files</h1>
    <ul>
        {% for file in files %}
        <li><a href="/trace/{{ file }}">{{ file }}</a></li>
        {% endfor %}
    </ul>
</body>
</html>
